# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('..'))


# -- Project information -----------------------------------------------------

project = 'cyminhook'
copyright = '2022, Segev Finer'
author = 'Segev Finer'

# The full version, including alpha/beta/rc tags
import cyminhook
release = cyminhook.__version__


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx',
    'sphinx.ext.githubpages'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'alabaster'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

html_theme_options = {
    'github_button': True,
    'github_user': 'segevfiner',
    'github_repo': 'cyminhook',
    'github_type': 'star',
    'extra_nav_links': {
        'cypcap @ PyPI': 'https://pypi.org/project/cyminhook/',
        'cypcap @ GitHub': 'https://github.com/segevfiner/cyminhook',
    }
}


# -- Options for intersphinx extension ---------------------------------------

intersphinx_mapping = {'python': ('https://docs.python.org/3', None)}


def remove_bpf_data_docstring(app, what, name, obj, options, lines):
    if what == 'data' and name.startswith("cypcap.bpf"):
        lines.clear()


def setup(app):
    app.connect("autodoc-process-docstring", remove_bpf_data_docstring)
