EXPORTS
    MH_Initialize
    MH_Uninitialize

    MH_CreateHook
    MH_CreateHookApi
    MH_CreateHookApiEx
    MH_RemoveHook
    MH_EnableHook
    MH_DisableHook
    MH_QueueEnableHook
    MH_QueueDisableHook
    MH_ApplyQueued
    MH_StatusToString
